@@ -42,44 +42,43 @@
 
 #include "nsBoxFrame.h"
 
-#include "gfxPrefs.h"
+#include <algorithm>
+#include <utility>
+
 #include "gfxUtils.h"
-#include "mozilla/gfx/2D.h"
-#include "nsBoxLayoutState.h"
-#include "mozilla/dom/Touch.h"
-#include "mozilla/Move.h"
 #include "mozilla/ComputedStyle.h"
+#include "mozilla/CSSOrderAwareFrameIterator.h"
+#include "mozilla/Preferences.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/dom/Touch.h"
+#include "mozilla/gfx/2D.h"
+#include "mozilla/gfx/gfxVars.h"
+#include "nsBoxLayout.h"
+#include "nsBoxLayoutState.h"
+#include "nsCOMPtr.h"
+#include "nsCSSAnonBoxes.h"
+#include "nsCSSRendering.h"
+#include "nsContainerFrame.h"
+#include "nsDisplayList.h"
+#include "nsGkAtoms.h"
+#include "nsHTMLParts.h"
+#include "nsIContent.h"
+#include "nsIFrameInlines.h"
+#include "nsIScrollableFrame.h"
+#include "nsITheme.h"
+#include "nsLayoutUtils.h"
+#include "nsNameSpaceManager.h"
 #include "nsPlaceholderFrame.h"
 #include "nsPresContext.h"
-#include "nsCOMPtr.h"
-#include "nsNameSpaceManager.h"
-#include "nsGkAtoms.h"
-#include "nsIContent.h"
-#include "nsHTMLParts.h"
-#include "nsViewManager.h"
-#include "nsView.h"
-#include "nsCSSRendering.h"
-#include "nsIServiceManager.h"
-#include "nsBoxLayout.h"
-#include "nsSprocketLayout.h"
-#include "nsIScrollableFrame.h"
-#include "nsWidgetsCID.h"
-#include "nsCSSAnonBoxes.h"
-#include "nsContainerFrame.h"
-#include "nsITheme.h"
-#include "nsTransform2D.h"
-#include "mozilla/EventStateManager.h"
-#include "mozilla/gfx/gfxVars.h"
-#include "nsDisplayList.h"
-#include "mozilla/Preferences.h"
-#include "nsStyleConsts.h"
-#include "nsLayoutUtils.h"
 #include "nsSliderFrame.h"
-#include <algorithm>
+#include "nsSprocketLayout.h"
+#include "nsStyleConsts.h"
+#include "nsTransform2D.h"
+#include "nsView.h"
+#include "nsViewManager.h"
+#include "nsWidgetsCID.h"
 
 // Needed for Print Preview
-#include "nsIURI.h"
 
 #include "mozilla/TouchEvents.h"
 
@@ -126,7 +125,32 @@ nsBoxFrame::nsBoxFrame(ComputedStyle* aStyle, nsPresContext* aPresContext,
   SetXULLayoutManager(layout);
 }
 
-nsBoxFrame::~nsBoxFrame() {}
+nsBoxFrame::~nsBoxFrame() = default;
+
+nsIFrame* nsBoxFrame::SlowOrdinalGroupAwareSibling(nsIFrame* aBox, bool aNext) {
+  nsIFrame* parent = aBox->GetParent();
+  if (!parent) {
+    return nullptr;
+  }
+  CSSOrderAwareFrameIterator iter(
+      parent, layout::kPrincipalList,
+      CSSOrderAwareFrameIterator::ChildFilter::IncludeAll,
+      CSSOrderAwareFrameIterator::OrderState::Unknown,
+      CSSOrderAwareFrameIterator::OrderingProperty::BoxOrdinalGroup);
+
+  nsIFrame* prevSibling = nullptr;
+  for (; !iter.AtEnd(); iter.Next()) {
+    nsIFrame* current = iter.get();
+    if (!aNext && current == aBox) {
+      return prevSibling;
+    }
+    if (aNext && prevSibling == aBox) {
+      return current;
+    }
+    prevSibling = current;
+  }
+  return nullptr;
+}
 
 void nsBoxFrame::SetInitialChildList(ChildListID aListID,
                                      nsFrameList& aChildList) {
@@ -134,7 +158,6 @@ void nsBoxFrame::SetInitialChildList(ChildListID aListID,
   if (aListID == kPrincipalList) {
     // initialize our list of infos.
     nsBoxLayoutState state(PresContext());
-    CheckBoxOrder();
     if (mLayoutManager)
       mLayoutManager->ChildrenSet(this, state, mFrames.FirstChild());
   }
@@ -156,37 +179,13 @@ void nsBoxFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
                       nsIFrame* aPrevInFlow) {
   nsContainerFrame::Init(aContent, aParent, aPrevInFlow);
 
-  if (GetStateBits() & NS_FRAME_FONT_INFLATION_CONTAINER) {
+  if (HasAnyStateBits(NS_FRAME_FONT_INFLATION_CONTAINER)) {
     AddStateBits(NS_FRAME_FONT_INFLATION_FLOW_ROOT);
   }
 
   MarkIntrinsicISizesDirty();
 
   CacheAttributes();
-
-  UpdateMouseThrough();
-
-  // register access key
-  RegUnregAccessKey(true);
-}
-
-void nsBoxFrame::UpdateMouseThrough() {
-  static Element::AttrValuesArray strings[] = {nsGkAtoms::never,
-                                               nsGkAtoms::always, nullptr};
-  switch (mContent->AsElement()->FindAttrValueIn(
-      kNameSpaceID_None, nsGkAtoms::mousethrough, strings, eCaseMatters)) {
-    case 0:
-      AddStateBits(NS_FRAME_MOUSE_THROUGH_NEVER);
-      break;
-    case 1:
-      AddStateBits(NS_FRAME_MOUSE_THROUGH_ALWAYS);
-      break;
-    case 2: {
-      RemoveStateBits(NS_FRAME_MOUSE_THROUGH_ALWAYS);
-      RemoveStateBits(NS_FRAME_MOUSE_THROUGH_NEVER);
-      break;
-    }
-  }
 }
 
 void nsBoxFrame::CacheAttributes() {
@@ -232,44 +231,10 @@ void nsBoxFrame::CacheAttributes() {
 }
 
 bool nsBoxFrame::GetInitialHAlignment(nsBoxFrame::Halignment& aHalign) {
-  if (!GetContent() || !GetContent()->IsElement()) return false;
+  if (!GetContent()) return false;
 
-  Element* element = GetContent()->AsElement();
-  // XXXdwh Everything inside this if statement is deprecated code.
-  static Element::AttrValuesArray alignStrings[] = {nsGkAtoms::left,
-                                                    nsGkAtoms::right, nullptr};
-  static const Halignment alignValues[] = {hAlign_Left, hAlign_Right};
-  int32_t index = element->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::align,
-                                           alignStrings, eCaseMatters);
-  if (index >= 0) {
-    aHalign = alignValues[index];
-    return true;
-  }
-
-  // Now that the deprecated stuff is out of the way, we move on to check the
-  // appropriate attribute.  For horizontal boxes, we are checking the PACK
-  // attribute.  For vertical boxes we are checking the ALIGN attribute.
-  nsAtom* attrName = IsXULHorizontal() ? nsGkAtoms::pack : nsGkAtoms::align;
-  static Element::AttrValuesArray strings[] = {
-      nsGkAtoms::_empty, nsGkAtoms::start, nsGkAtoms::center, nsGkAtoms::end,
-      nullptr};
-  static const Halignment values[] = {hAlign_Left /*not used*/, hAlign_Left,
-                                      hAlign_Center, hAlign_Right};
-  index = element->FindAttrValueIn(kNameSpaceID_None, attrName, strings,
-                                   eCaseMatters);
-
-  if (index == Element::ATTR_VALUE_NO_MATCH) {
-    // The attr was present but had a nonsensical value. Revert to the default.
-    return false;
-  }
-  if (index > 0) {
-    aHalign = values[index];
-    return true;
-  }
-
-  // Now that we've checked for the attribute it's time to check CSS.  For
-  // horizontal boxes we're checking PACK.  For vertical boxes we are checking
-  // ALIGN.
+  // For horizontal boxes we're checking PACK.  For vertical boxes we are
+  // checking ALIGN.
   const nsStyleXUL* boxInfo = StyleXUL();
   if (IsXULHorizontal()) {
     switch (boxInfo->mBoxPack) {
@@ -305,46 +270,9 @@ bool nsBoxFrame::GetInitialHAlignment(nsBoxFrame::Halignment& aHalign) {
 }
 
 bool nsBoxFrame::GetInitialVAlignment(nsBoxFrame::Valignment& aValign) {
-  if (!GetContent() || !GetContent()->IsElement()) return false;
-
-  Element* element = GetContent()->AsElement();
-
-  static Element::AttrValuesArray valignStrings[] = {
-      nsGkAtoms::top, nsGkAtoms::baseline, nsGkAtoms::middle, nsGkAtoms::bottom,
-      nullptr};
-  static const Valignment valignValues[] = {vAlign_Top, vAlign_BaseLine,
-                                            vAlign_Middle, vAlign_Bottom};
-  int32_t index = element->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::valign,
-                                           valignStrings, eCaseMatters);
-  if (index >= 0) {
-    aValign = valignValues[index];
-    return true;
-  }
-
-  // Now that the deprecated stuff is out of the way, we move on to check the
-  // appropriate attribute.  For horizontal boxes, we are checking the ALIGN
-  // attribute.  For vertical boxes we are checking the PACK attribute.
-  nsAtom* attrName = IsXULHorizontal() ? nsGkAtoms::align : nsGkAtoms::pack;
-  static Element::AttrValuesArray strings[] = {
-      nsGkAtoms::_empty,   nsGkAtoms::start, nsGkAtoms::center,
-      nsGkAtoms::baseline, nsGkAtoms::end,   nullptr};
-  static const Valignment values[] = {vAlign_Top /*not used*/, vAlign_Top,
-                                      vAlign_Middle, vAlign_BaseLine,
-                                      vAlign_Bottom};
-  index = element->FindAttrValueIn(kNameSpaceID_None, attrName, strings,
-                                   eCaseMatters);
-  if (index == Element::ATTR_VALUE_NO_MATCH) {
-    // The attr was present but had a nonsensical value. Revert to the default.
-    return false;
-  }
-  if (index > 0) {
-    aValign = values[index];
-    return true;
-  }
-
-  // Now that we've checked for the attribute it's time to check CSS.  For
-  // horizontal boxes we're checking ALIGN.  For vertical boxes we are checking
-  // PACK.
+  if (!GetContent()) return false;
+  // For horizontal boxes we're checking ALIGN.  For vertical boxes we are
+  // checking PACK.
   const nsStyleXUL* boxInfo = StyleXUL();
   if (IsXULHorizontal()) {
     switch (boxInfo->mBoxAlign) {
@@ -386,25 +314,12 @@ void nsBoxFrame::GetInitialOrientation(bool& aIsHorizontal) {
   // see if we are a vertical or horizontal box.
   if (!GetContent()) return;
 
-  // Check the style system first.
   const nsStyleXUL* boxInfo = StyleXUL();
   if (boxInfo->mBoxOrient == StyleBoxOrient::Horizontal) {
     aIsHorizontal = true;
   } else {
     aIsHorizontal = false;
   }
-
-  // Now see if we have an attribute.  The attribute overrides
-  // the style system value.
-  if (!GetContent()->IsElement()) return;
-
-  static Element::AttrValuesArray strings[] = {nsGkAtoms::vertical,
-                                               nsGkAtoms::horizontal, nullptr};
-  int32_t index = GetContent()->AsElement()->FindAttrValueIn(
-      kNameSpaceID_None, nsGkAtoms::orient, strings, eCaseMatters);
-  if (index >= 0) {
-    aIsHorizontal = index == 1;
-  }
 }
 
 void nsBoxFrame::GetInitialDirection(bool& aIsNormal) {
@@ -414,38 +329,32 @@ void nsBoxFrame::GetInitialDirection(bool& aIsNormal) {
     // For horizontal boxes only, we initialize our value based off the CSS
     // 'direction' property. This means that BiDI users will end up with
     // horizontally inverted chrome.
-    aIsNormal = (StyleVisibility()->mDirection ==
-                 NS_STYLE_DIRECTION_LTR);  // If text runs RTL then so do we.
-  } else
+    //
+    // If text runs RTL then so do we.
+    aIsNormal = StyleVisibility()->mDirection == StyleDirection::Ltr;
+    if (GetContent()->IsElement()) {
+      Element* element = GetContent()->AsElement();
+
+      // Now see if we have an attribute. The attribute overrides
+      // the style system 'direction' property.
+      static Element::AttrValuesArray strings[] = {nsGkAtoms::ltr,
+                                                   nsGkAtoms::rtl, nullptr};
+      int32_t index = element->FindAttrValueIn(
+          kNameSpaceID_None, nsGkAtoms::dir, strings, eCaseMatters);
+      if (index >= 0) {
+        bool values[] = {true, false};
+        aIsNormal = values[index];
+      }
+    }
+  } else {
     aIsNormal = true;  // Assume a normal direction in the vertical case.
+  }
 
   // Now check the style system to see if we should invert aIsNormal.
   const nsStyleXUL* boxInfo = StyleXUL();
   if (boxInfo->mBoxDirection == StyleBoxDirection::Reverse) {
     aIsNormal = !aIsNormal;  // Invert our direction.
   }
-
-  if (!GetContent()->IsElement()) {
-    return;
-  }
-
-  Element* element = GetContent()->AsElement();
-
-  // Now see if we have an attribute.  The attribute overrides
-  // the style system value.
-  if (IsXULHorizontal()) {
-    static Element::AttrValuesArray strings[] = {
-        nsGkAtoms::reverse, nsGkAtoms::ltr, nsGkAtoms::rtl, nullptr};
-    int32_t index = element->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::dir,
-                                             strings, eCaseMatters);
-    if (index >= 0) {
-      bool values[] = {!aIsNormal, true, false};
-      aIsNormal = values[index];
-    }
-  } else if (element->AttrValueIs(kNameSpaceID_None, nsGkAtoms::dir,
-                                  nsGkAtoms::reverse, eCaseMatters)) {
-    aIsNormal = !aIsNormal;
-  }
 }
 
 /* Returns true if it was set.
@@ -469,18 +378,6 @@ bool nsBoxFrame::GetInitialEqualSize(bool& aEqualSize) {
 bool nsBoxFrame::GetInitialAutoStretch(bool& aStretch) {
   if (!GetContent()) return false;
 
-  // Check the align attribute.
-  if (GetContent()->IsElement()) {
-    static Element::AttrValuesArray strings[] = {nsGkAtoms::_empty,
-                                                 nsGkAtoms::stretch, nullptr};
-    int32_t index = GetContent()->AsElement()->FindAttrValueIn(
-        kNameSpaceID_None, nsGkAtoms::align, strings, eCaseMatters);
-    if (index != Element::ATTR_MISSING && index != 0) {
-      aStretch = index == 1;
-      return true;
-    }
-  }
-
   // Check the CSS box-align property.
   const nsStyleXUL* boxInfo = StyleXUL();
   aStretch = (boxInfo->mBoxAlign == StyleBoxAlign::Stretch);
@@ -492,11 +389,14 @@ void nsBoxFrame::DidReflow(nsPresContext* aPresContext,
                            const ReflowInput* aReflowInput) {
   nsFrameState preserveBits =
       mState & (NS_FRAME_IS_DIRTY | NS_FRAME_HAS_DIRTY_CHILDREN);
-  nsFrame::DidReflow(aPresContext, aReflowInput);
+  nsIFrame::DidReflow(aPresContext, aReflowInput);
   AddStateBits(preserveBits);
+  if (preserveBits & NS_FRAME_IS_DIRTY) {
+    this->MarkSubtreeDirty();
+  }
 }
 
-bool nsBoxFrame::HonorPrintBackgroundSettings() {
+bool nsBoxFrame::HonorPrintBackgroundSettings() const {
   return !mContent->IsInNativeAnonymousSubtree() &&
          nsContainerFrame::HonorPrintBackgroundSettings();
 }
@@ -591,31 +491,31 @@ void nsBoxFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aDesiredSize,
   WritingMode wm = aReflowInput.GetWritingMode();
   LogicalSize computedSize = aReflowInput.ComputedSize();
 
-  LogicalMargin m = aReflowInput.ComputedLogicalBorderPadding();
+  LogicalMargin m = aReflowInput.ComputedLogicalBorderPadding(wm);
   // GetXULBorderAndPadding(m);
 
   LogicalSize prefSize(wm);
 
   // if we are told to layout intrinsic then get our preferred size.
-  NS_ASSERTION(computedSize.ISize(wm) != NS_INTRINSICSIZE,
+  NS_ASSERTION(computedSize.ISize(wm) != NS_UNCONSTRAINEDSIZE,
                "computed inline size should always be computed");
-  if (computedSize.BSize(wm) == NS_INTRINSICSIZE) {
+  if (computedSize.BSize(wm) == NS_UNCONSTRAINEDSIZE) {
     nsSize physicalPrefSize = GetXULPrefSize(state);
     nsSize minSize = GetXULMinSize(state);
     nsSize maxSize = GetXULMaxSize(state);
     // XXXbz isn't GetXULPrefSize supposed to bounds-check for us?
-    physicalPrefSize = BoundsCheck(minSize, physicalPrefSize, maxSize);
+    physicalPrefSize = XULBoundsCheck(minSize, physicalPrefSize, maxSize);
     prefSize = LogicalSize(wm, physicalPrefSize);
   }
 
   // get our desiredSize
   computedSize.ISize(wm) += m.IStart(wm) + m.IEnd(wm);
 
-  if (aReflowInput.ComputedBSize() == NS_INTRINSICSIZE) {
+  if (aReflowInput.ComputedBSize() == NS_UNCONSTRAINEDSIZE) {
     computedSize.BSize(wm) = prefSize.BSize(wm);
     // prefSize is border-box but min/max constraints are content-box.
     nscoord blockDirBorderPadding =
-        aReflowInput.ComputedLogicalBorderPadding().BStartEnd(wm);
+        aReflowInput.ComputedLogicalBorderPadding(wm).BStartEnd(wm);
     nscoord contentBSize = computedSize.BSize(wm) - blockDirBorderPadding;
     // Note: contentHeight might be negative, but that's OK because min-height
     // is never negative.
@@ -674,7 +574,7 @@ nsSize nsBoxFrame::GetXULPrefSize(nsBoxLayoutState& aBoxLayoutState) {
 
   nsSize size(0, 0);
   DISPLAY_PREF_SIZE(this, size);
-  if (!DoesNeedRecalc(mPrefSize)) {
+  if (!XULNeedsRecalc(mPrefSize)) {
     size = mPrefSize;
     return size;
   }
@@ -689,26 +589,31 @@ nsSize nsBoxFrame::GetXULPrefSize(nsBoxLayoutState& aBoxLayoutState) {
       if (!widthSet) size.width = layoutSize.width;
       if (!heightSet) size.height = layoutSize.height;
     } else {
-      size = nsBox::GetXULPrefSize(aBoxLayoutState);
+      size = nsIFrame::GetUncachedXULPrefSize(aBoxLayoutState);
     }
   }
 
   nsSize minSize = GetXULMinSize(aBoxLayoutState);
   nsSize maxSize = GetXULMaxSize(aBoxLayoutState);
-  mPrefSize = BoundsCheck(minSize, size, maxSize);
+  mPrefSize = XULBoundsCheck(minSize, size, maxSize);
 
   return mPrefSize;
 }
 
 nscoord nsBoxFrame::GetXULBoxAscent(nsBoxLayoutState& aBoxLayoutState) {
-  if (!DoesNeedRecalc(mAscent)) return mAscent;
+  if (!XULNeedsRecalc(mAscent)) {
+    return mAscent;
+  }
 
-  if (IsXULCollapsed()) return 0;
+  if (IsXULCollapsed()) {
+    return 0;
+  }
 
-  if (mLayoutManager)
+  if (mLayoutManager) {
     mAscent = mLayoutManager->GetAscent(this, aBoxLayoutState);
-  else
-    mAscent = nsBox::GetXULBoxAscent(aBoxLayoutState);
+  } else {
+    mAscent = GetXULPrefSize(aBoxLayoutState).height;
+  }
 
   return mAscent;
 }
@@ -719,7 +624,7 @@ nsSize nsBoxFrame::GetXULMinSize(nsBoxLayoutState& aBoxLayoutState) {
 
   nsSize size(0, 0);
   DISPLAY_MIN_SIZE(this, size);
-  if (!DoesNeedRecalc(mMinSize)) {
+  if (!XULNeedsRecalc(mMinSize)) {
     size = mMinSize;
     return size;
   }
@@ -728,14 +633,13 @@ nsSize nsBoxFrame::GetXULMinSize(nsBoxLayoutState& aBoxLayoutState) {
 
   // if the size was not completely redefined in CSS then ask our children
   bool widthSet, heightSet;
-  if (!nsIFrame::AddXULMinSize(aBoxLayoutState, this, size, widthSet,
-                               heightSet)) {
+  if (!nsIFrame::AddXULMinSize(this, size, widthSet, heightSet)) {
     if (mLayoutManager) {
       nsSize layoutSize = mLayoutManager->GetXULMinSize(this, aBoxLayoutState);
       if (!widthSet) size.width = layoutSize.width;
       if (!heightSet) size.height = layoutSize.height;
     } else {
-      size = nsBox::GetXULMinSize(aBoxLayoutState);
+      size = nsIFrame::GetUncachedXULMinSize(aBoxLayoutState);
     }
   }
 
@@ -748,9 +652,9 @@ nsSize nsBoxFrame::GetXULMaxSize(nsBoxLayoutState& aBoxLayoutState) {
   NS_ASSERTION(aBoxLayoutState.GetRenderingContext(),
                "must have rendering context");
 
-  nsSize size(NS_INTRINSICSIZE, NS_INTRINSICSIZE);
+  nsSize size(NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE);
   DISPLAY_MAX_SIZE(this, size);
-  if (!DoesNeedRecalc(mMaxSize)) {
+  if (!XULNeedsRecalc(mMaxSize)) {
     size = mMaxSize;
     return size;
   }
@@ -765,7 +669,7 @@ nsSize nsBoxFrame::GetXULMaxSize(nsBoxLayoutState& aBoxLayoutState) {
       if (!widthSet) size.width = layoutSize.width;
       if (!heightSet) size.height = layoutSize.height;
     } else {
-      size = nsBox::GetXULMaxSize(aBoxLayoutState);
+      size = nsIFrame::GetUncachedXULMaxSize(aBoxLayoutState);
     }
   }
 
@@ -775,9 +679,9 @@ nsSize nsBoxFrame::GetXULMaxSize(nsBoxLayoutState& aBoxLayoutState) {
 }
 
 nscoord nsBoxFrame::GetXULFlex() {
-  if (!DoesNeedRecalc(mFlex)) return mFlex;
-
-  mFlex = nsBox::GetXULFlex();
+  if (XULNeedsRecalc(mFlex)) {
+    nsIFrame::AddXULFlex(this, mFlex);
+  }
 
   return mFlex;
 }
@@ -788,12 +692,12 @@ nscoord nsBoxFrame::GetXULFlex() {
  */
 NS_IMETHODIMP
 nsBoxFrame::DoXULLayout(nsBoxLayoutState& aState) {
-  uint32_t oldFlags = aState.LayoutFlags();
-  aState.SetLayoutFlags(0);
+  ReflowChildFlags oldFlags = aState.LayoutFlags();
+  aState.SetLayoutFlags(ReflowChildFlags::Default);
 
   nsresult rv = NS_OK;
   if (mLayoutManager) {
-    CoordNeedsRecalc(mAscent);
+    XULCoordNeedsRecalc(mAscent);
     rv = mLayoutManager->XULLayout(this, aState);
   }
 
@@ -836,9 +740,6 @@ nsBoxFrame::DoXULLayout(nsBoxLayoutState& aState) {
 
 void nsBoxFrame::DestroyFrom(nsIFrame* aDestructRoot,
                              PostDestroyData& aPostDestroyData) {
-  // unregister access key
-  RegUnregAccessKey(false);
-
   // clean up the container box's layout manager and child boxes
   SetXULLayoutManager(nullptr);
 
@@ -847,11 +748,11 @@ void nsBoxFrame::DestroyFrom(nsIFrame* aDestructRoot,
 
 /* virtual */
 void nsBoxFrame::MarkIntrinsicISizesDirty() {
-  SizeNeedsRecalc(mPrefSize);
-  SizeNeedsRecalc(mMinSize);
-  SizeNeedsRecalc(mMaxSize);
-  CoordNeedsRecalc(mFlex);
-  CoordNeedsRecalc(mAscent);
+  XULSizeNeedsRecalc(mPrefSize);
+  XULSizeNeedsRecalc(mMinSize);
+  XULSizeNeedsRecalc(mMaxSize);
+  XULCoordNeedsRecalc(mFlex);
+  XULCoordNeedsRecalc(mAscent);
 
   if (mLayoutManager) {
     nsBoxLayoutState state(PresContext());
@@ -882,6 +783,7 @@ void nsBoxFrame::RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) {
 }
 
 void nsBoxFrame::InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
+                              const nsLineList::iterator* aPrevFrameLine,
                               nsFrameList& aFrameList) {
   NS_ASSERTION(!aPrevFrame || aPrevFrame->GetParent() == this,
                "inserting after sibling frame with different parent");
@@ -899,12 +801,6 @@ void nsBoxFrame::InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
   if (mLayoutManager)
     mLayoutManager->ChildrenInserted(this, state, aPrevFrame, newFrames);
 
-  // Make sure to check box order _after_ notifying the layout
-  // manager; otherwise the slice we give the layout manager will
-  // just be bogus.  If the layout manager cares about the order, we
-  // just lose.
-  CheckBoxOrder();
-
   PresShell()->FrameNeedsReflow(this, IntrinsicDirty::TreeChange,
                                 NS_FRAME_HAS_DIRTY_CHILDREN);
 }
@@ -920,14 +816,8 @@ void nsBoxFrame::AppendFrames(ChildListID aListID, nsFrameList& aFrameList) {
   // notify the layout manager
   if (mLayoutManager) mLayoutManager->ChildrenAppended(this, state, newFrames);
 
-  // Make sure to check box order _after_ notifying the layout
-  // manager; otherwise the slice we give the layout manager will
-  // just be bogus.  If the layout manager cares about the order, we
-  // just lose.
-  CheckBoxOrder();
-
   // XXXbz why is this NS_FRAME_FIRST_REFLOW check here?
-  if (!(GetStateBits() & NS_FRAME_FIRST_REFLOW)) {
+  if (!HasAnyStateBits(NS_FRAME_FIRST_REFLOW)) {
     PresShell()->FrameNeedsReflow(this, IntrinsicDirty::TreeChange,
                                   NS_FRAME_HAS_DIRTY_CHILDREN);
   }
@@ -935,8 +825,9 @@ void nsBoxFrame::AppendFrames(ChildListID aListID, nsFrameList& aFrameList) {
 
 /* virtual */
 nsContainerFrame* nsBoxFrame::GetContentInsertionFrame() {
-  if (GetStateBits() & NS_STATE_BOX_WRAPS_KIDS_IN_BLOCK)
+  if (HasAnyStateBits(NS_STATE_BOX_WRAPS_KIDS_IN_BLOCK)) {
     return PrincipalChildList().FirstChild()->GetContentInsertionFrame();
+  }
   return nsContainerFrame::GetContentInsertionFrame();
 }
 
@@ -947,8 +838,7 @@ nsresult nsBoxFrame::AttributeChanged(int32_t aNameSpaceID, nsAtom* aAttribute,
 
   // Ignore 'width', 'height', 'screenX', 'screenY' and 'sizemode' on a
   // <window>.
-  if (mContent->IsAnyOfXULElements(nsGkAtoms::window, nsGkAtoms::page,
-                                   nsGkAtoms::dialog, nsGkAtoms::wizard) &&
+  if (mContent->IsXULElement(nsGkAtoms::window) &&
       (nsGkAtoms::width == aAttribute || nsGkAtoms::height == aAttribute ||
        nsGkAtoms::screenX == aAttribute || nsGkAtoms::screenY == aAttribute ||
        nsGkAtoms::sizemode == aAttribute)) {
@@ -957,15 +847,11 @@ nsresult nsBoxFrame::AttributeChanged(int32_t aNameSpaceID, nsAtom* aAttribute,
 
   if (aAttribute == nsGkAtoms::width || aAttribute == nsGkAtoms::height ||
       aAttribute == nsGkAtoms::align || aAttribute == nsGkAtoms::valign ||
-      aAttribute == nsGkAtoms::left || aAttribute == nsGkAtoms::top ||
-      aAttribute == nsGkAtoms::right || aAttribute == nsGkAtoms::bottom ||
-      aAttribute == nsGkAtoms::start || aAttribute == nsGkAtoms::end ||
       aAttribute == nsGkAtoms::minwidth || aAttribute == nsGkAtoms::maxwidth ||
       aAttribute == nsGkAtoms::minheight ||
       aAttribute == nsGkAtoms::maxheight || aAttribute == nsGkAtoms::flex ||
       aAttribute == nsGkAtoms::orient || aAttribute == nsGkAtoms::pack ||
-      aAttribute == nsGkAtoms::dir || aAttribute == nsGkAtoms::mousethrough ||
-      aAttribute == nsGkAtoms::equalsize) {
+      aAttribute == nsGkAtoms::dir || aAttribute == nsGkAtoms::equalsize) {
     if (aAttribute == nsGkAtoms::align || aAttribute == nsGkAtoms::valign ||
         aAttribute == nsGkAtoms::orient || aAttribute == nsGkAtoms::pack ||
         aAttribute == nsGkAtoms::dir) {
@@ -1002,36 +888,10 @@ nsresult nsBoxFrame::AttributeChanged(int32_t aNameSpaceID, nsAtom* aAttribute,
         AddStateBits(NS_STATE_AUTO_STRETCH);
       else
         RemoveStateBits(NS_STATE_AUTO_STRETCH);
-    } else if (aAttribute == nsGkAtoms::left || aAttribute == nsGkAtoms::top ||
-               aAttribute == nsGkAtoms::right ||
-               aAttribute == nsGkAtoms::bottom ||
-               aAttribute == nsGkAtoms::start || aAttribute == nsGkAtoms::end) {
-      RemoveStateBits(NS_STATE_STACK_NOT_POSITIONED);
-    } else if (aAttribute == nsGkAtoms::mousethrough) {
-      UpdateMouseThrough();
     }
 
     PresShell()->FrameNeedsReflow(this, IntrinsicDirty::StyleChange,
                                   NS_FRAME_IS_DIRTY);
-  } else if (aAttribute == nsGkAtoms::ordinal) {
-    nsIFrame* parent = GetParentXULBox(this);
-    // If our parent is not a box, there's not much we can do... but in that
-    // case our ordinal doesn't matter anyway, so that's ok.
-    // Also don't bother with popup frames since they are kept on the
-    // kPopupList and XULRelayoutChildAtOrdinal() only handles
-    // principal children.
-    if (parent && !(GetStateBits() & NS_FRAME_OUT_OF_FLOW) &&
-        StyleDisplay()->mDisplay != mozilla::StyleDisplay::MozPopup) {
-      parent->XULRelayoutChildAtOrdinal(this);
-      // XXXldb Should this instead be a tree change on the child or parent?
-      PresShell()->FrameNeedsReflow(parent, IntrinsicDirty::StyleChange,
-                                    NS_FRAME_IS_DIRTY);
-    }
-  }
-  // If the accesskey changed, register for the new value
-  // The old value has been unregistered in nsXULElement::SetAttr
-  else if (aAttribute == nsGkAtoms::accesskey) {
-    RegUnregAccessKey(true);
   } else if (aAttribute == nsGkAtoms::rows &&
              mContent->IsXULElement(nsGkAtoms::tree)) {
     // Reflow ourselves and all our children if "rows" changes, since
@@ -1046,13 +906,6 @@ nsresult nsBoxFrame::AttributeChanged(int32_t aNameSpaceID, nsAtom* aAttribute,
 void nsBoxFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
                                   const nsDisplayListSet& aLists) {
   bool forceLayer = false;
-  // We check the renderroot attribute here in nsBoxFrame for lack of a better
-  // place. This roughly mirrors the pre-existing "layer" attribute. In the
-  // long term we may want to add a specific element in which we can wrap
-  // alternate renderroot content, but we're electing to not go down that
-  // rabbit hole today.
-  wr::RenderRoot renderRoot =
-      gfxUtils::GetRenderRootForFrame(this).valueOr(wr::RenderRoot::Default);
 
   if (GetContent()->IsXULElement()) {
     // forcelayer is only supported on XUL elements with box layout
@@ -1063,21 +916,20 @@ void nsBoxFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
     // Check for frames that are marked as a part of the region used
     // in calculating glass margins on Windows.
     const nsStyleDisplay* styles = StyleDisplay();
-    if (styles && styles->mAppearance == StyleAppearance::MozWinExcludeGlass) {
+    if (styles &&
+        styles->EffectiveAppearance() == StyleAppearance::MozWinExcludeGlass) {
       aBuilder->AddWindowExcludeGlassRegion(
           this, nsRect(aBuilder->ToReferenceFrame(this), GetSize()));
     }
   }
 
   nsDisplayListCollection tempLists(aBuilder);
-  const nsDisplayListSet& destination =
-      (forceLayer || renderRoot != wr::RenderRoot::Default) ? tempLists
-                                                            : aLists;
+  const nsDisplayListSet& destination = (forceLayer) ? tempLists : aLists;
 
   DisplayBorderBackgroundOutline(aBuilder, destination);
 
   Maybe<nsDisplayListBuilder::AutoContainerASRTracker> contASRTracker;
-  if (forceLayer || renderRoot != wr::RenderRoot::Default) {
+  if (forceLayer) {
     contASRTracker.emplace(aBuilder);
   }
 
@@ -1086,7 +938,7 @@ void nsBoxFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
   // see if we have to draw a selection frame around this container
   DisplaySelectionOverlay(aBuilder, destination.Content());
 
-  if (forceLayer || renderRoot != wr::RenderRoot::Default) {
+  if (forceLayer) {
     // This is a bit of a hack. Collect up all descendant display items
     // and merge them into a single Content() list. This can cause us
     // to violate CSS stacking order, but forceLayer is a magic
@@ -1101,97 +953,49 @@ void nsBoxFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
     const ActiveScrolledRoot* ownLayerASR = contASRTracker->GetContainerASR();
     DisplayListClipState::AutoSaveRestore ownLayerClipState(aBuilder);
 
-    if (forceLayer) {
-      MOZ_ASSERT(renderRoot == wr::RenderRoot::Default);
-      // Wrap the list to make it its own layer
-      aLists.Content()->AppendNewToTop<nsDisplayOwnLayer>(
-          aBuilder, this, &masterList, ownLayerASR,
-          nsDisplayOwnLayerFlags::None, mozilla::layers::ScrollbarData{}, true,
-          true);
-    } else {
-      MOZ_ASSERT(!XRE_IsContentProcess());
-      aLists.Content()->AppendNewToTop<nsDisplayRenderRoot>(
-          aBuilder, this, &masterList, ownLayerASR, renderRoot);
-    }
+    // Wrap the list to make it its own layer
+    aLists.Content()->AppendNewToTopWithIndex<nsDisplayOwnLayer>(
+        aBuilder, this, /* aIndex = */ nsDisplayOwnLayer::OwnLayerForBoxFrame,
+        &masterList, ownLayerASR, nsDisplayOwnLayerFlags::None,
+        mozilla::layers::ScrollbarData{}, true, true);
   }
 }
 
 void nsBoxFrame::BuildDisplayListForChildren(nsDisplayListBuilder* aBuilder,
                                              const nsDisplayListSet& aLists) {
-  nsIFrame* kid = mFrames.FirstChild();
+  // Iterate over the children in CSS order.
+  auto iter = CSSOrderAwareFrameIterator(
+      this, mozilla::layout::kPrincipalList,
+      CSSOrderAwareFrameIterator::ChildFilter::IncludeAll,
+      CSSOrderAwareFrameIterator::OrderState::Unknown,
+      CSSOrderAwareFrameIterator::OrderingProperty::BoxOrdinalGroup);
   // Put each child's background onto the BlockBorderBackgrounds list
   // to emulate the existing two-layer XUL painting scheme.
   nsDisplayListSet set(aLists, aLists.BlockBorderBackgrounds());
-  // The children should be in the right order
-  while (kid) {
-    BuildDisplayListForChild(aBuilder, kid, set);
-    kid = kid->GetNextSibling();
+  for (; !iter.AtEnd(); iter.Next()) {
+    BuildDisplayListForChild(aBuilder, iter.get(), set);
   }
 }
 
 #ifdef DEBUG_FRAME_DUMP
 nsresult nsBoxFrame::GetFrameName(nsAString& aResult) const {
-  return MakeFrameName(NS_LITERAL_STRING("Box"), aResult);
+  return MakeFrameName(u"Box"_ns, aResult);
 }
 #endif
 
-// If you make changes to this function, check its counterparts
-// in nsTextBoxFrame and nsXULLabelFrame
-void nsBoxFrame::RegUnregAccessKey(bool aDoReg) {
-  MOZ_ASSERT(mContent);
-
-  // only support accesskeys for the following elements
-  if (!mContent->IsAnyOfXULElements(nsGkAtoms::button, nsGkAtoms::toolbarbutton,
-                                    nsGkAtoms::checkbox, nsGkAtoms::textbox,
-                                    nsGkAtoms::tab, nsGkAtoms::radio)) {
-    return;
-  }
-
-  nsAutoString accessKey;
-  mContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::accesskey,
-                                 accessKey);
-
-  if (accessKey.IsEmpty()) return;
-
-  // With a valid PresContext we can get the ESM
-  // and register the access key
-  EventStateManager* esm = PresContext()->EventStateManager();
-
-  uint32_t key = accessKey.First();
-  if (aDoReg)
-    esm->RegisterAccessKey(mContent->AsElement(), key);
-  else
-    esm->UnregisterAccessKey(mContent->AsElement(), key);
-}
-
 void nsBoxFrame::AppendDirectlyOwnedAnonBoxes(nsTArray<OwnedAnonBox>& aResult) {
-  if (GetStateBits() & NS_STATE_BOX_WRAPS_KIDS_IN_BLOCK) {
+  if (HasAnyStateBits(NS_STATE_BOX_WRAPS_KIDS_IN_BLOCK)) {
     aResult.AppendElement(OwnedAnonBox(PrincipalChildList().FirstChild()));
   }
 }
 
-// Helper less-than-or-equal function, used in CheckBoxOrder() as a
-// template-parameter for the sorting functions.
-static bool IsBoxOrdinalLEQ(nsIFrame* aFrame1, nsIFrame* aFrame2) {
-  // If we've got a placeholder frame, use its out-of-flow frame's ordinal val.
-  nsIFrame* aRealFrame1 = nsPlaceholderFrame::GetRealFrameFor(aFrame1);
-  nsIFrame* aRealFrame2 = nsPlaceholderFrame::GetRealFrameFor(aFrame2);
-  return aRealFrame1->GetXULOrdinal() <= aRealFrame2->GetXULOrdinal();
-}
-
-void nsBoxFrame::CheckBoxOrder() {
-  if (!nsIFrame::IsFrameListSorted<IsBoxOrdinalLEQ>(mFrames)) {
-    nsIFrame::SortFrameList<IsBoxOrdinalLEQ>(mFrames);
-  }
-}
-
 nsresult nsBoxFrame::LayoutChildAt(nsBoxLayoutState& aState, nsIFrame* aBox,
                                    const nsRect& aRect) {
   // get the current rect
   nsRect oldRect(aBox->GetRect());
   aBox->SetXULBounds(aState, aRect);
 
-  bool layout = NS_SUBTREE_DIRTY(aBox);
+  bool layout = aBox->IsSubtreeDirty();
 
   if (layout ||
       (oldRect.width != aRect.width || oldRect.height != aRect.height)) {
@@ -1201,38 +1005,6 @@ nsresult nsBoxFrame::LayoutChildAt(nsBoxLayoutState& aState, nsIFrame* aBox,
   return NS_OK;
 }
 
-nsresult nsBoxFrame::XULRelayoutChildAtOrdinal(nsIFrame* aChild) {
-  uint32_t ord = aChild->GetXULOrdinal();
-
-  nsIFrame* child = mFrames.FirstChild();
-  nsIFrame* newPrevSib = nullptr;
-
-  while (child) {
-    if (ord < child->GetXULOrdinal()) {
-      break;
-    }
-
-    if (child != aChild) {
-      newPrevSib = child;
-    }
-
-    child = GetNextXULBox(child);
-  }
-
-  if (aChild->GetPrevSibling() == newPrevSib) {
-    // This box is not moving.
-    return NS_OK;
-  }
-
-  // Take |aChild| out of its old position in the child list.
-  mFrames.RemoveFrame(aChild);
-
-  // Insert it after |newPrevSib| or at the start if it's null.
-  mFrames.InsertFrame(nullptr, newPrevSib, aChild);
-
-  return NS_OK;
-}
-
 /**
  * This wrapper class lets us redirect mouse hits from descendant frames
  * of a menu to the menu itself, if they didn't specify 'allowevents'.
@@ -1271,6 +1043,10 @@ class nsDisplayXULEventRedirector final : public nsDisplayWrapList {
   virtual bool ShouldFlattenAway(nsDisplayListBuilder* aBuilder) override {
     return false;
   }
+  void Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) override {
+    GetChildren()->Paint(aBuilder, aCtx,
+                         mFrame->PresContext()->AppUnitsPerDevPixel());
+  }
   NS_DISPLAY_DECL_NAME("XULEventRedirector", TYPE_XUL_EVENT_REDIRECTOR)
  private:
   nsIFrame* mTargetFrame;
@@ -1311,7 +1087,7 @@ void nsDisplayXULEventRedirector::HitTest(nsDisplayListBuilder* aBuilder,
   }
 }
 
-class nsXULEventRedirectorWrapper final : public nsDisplayWrapper {
+class nsXULEventRedirectorWrapper final : public nsDisplayItemWrapper {
  public:
   explicit nsXULEventRedirectorWrapper(nsIFrame* aTargetFrame)
       : mTargetFrame(aTargetFrame) {}
@@ -1341,7 +1117,8 @@ void nsBoxFrame::WrapListsInRedirector(nsDisplayListBuilder* aBuilder,
 bool nsBoxFrame::GetEventPoint(WidgetGUIEvent* aEvent, nsPoint& aPoint) {
   LayoutDeviceIntPoint refPoint;
   bool res = GetEventPoint(aEvent, refPoint);
-  aPoint = nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent, refPoint, this);
+  aPoint = nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent, refPoint,
+                                                        RelativeTo{this});
   return res;
 }
 
